﻿# coding: utf-8
import sys
import os
import cookielib, urllib, urllib2
import re
import datetime

class Result:
    def __init__(self, date):
        self.date = date
        self.entries = []

    def get_entry(self, id):
        for e in self.entries:
            if e.id == id:
                return e
        return None

    def resolve(self):
        # generatorをすべて処理完了する。
        self.entries = list(self.entries)

class Entry:
    def __init__(self, id, name, ev=None, good=None, mid=None, bad=None):
        self.id = id
        self.name = name
        if ev:
            self.good = ev[u'良かった']
            self.mid = ev[u'普通']
            self.bad = ev[u'イマイチ']
        else:
            self.good = good 
            self.mid = mid 
            self.bad = bad 

    def get_entry(self, id):
        for e in self.entries:
            if e.id == id:
                return e
        return None

    def __str__(self):
        return self.__repr__()

    def __repr__(self):
        return ('Entry(id=' + repr(self.id) +
                ', name=' + repr(self.name) + 
                ', good=' + repr(self.good) + 
                ', mid=' + repr(self.mid) + 
                ', bad=' + repr(self.bad) + ')')

    
class ObLoveSite:
    def __init__(self, baseurl):
        self.cj = cookielib.CookieJar()
        self.opener = urllib2.build_opener(urllib2.HTTPCookieProcessor(self.cj))
        self.baseurl = baseurl
        self.logged_in = False

    def login(self, user, password):
        params = urllib.urlencode({'__ac_name': user, '__ac_password': password, 'came_from': ""})
        headers = {"Content-type": "application/x-www-form-urlencoded",
                   "Accept": "text/plain"}

        req = urllib2.Request(url=self.baseurl + "/logged_in", data=params)
        f = self.opener.open(req)
        for l in f:
            pass
        f.close()
        self.logged_in = True

    def access(self, url):
        f = self.opener.open(self.baseurl + url)
        page = ""
        for l in f: page += l
        return unicode(page, 'utf-8')

    def list_category_urls(self):
        page = self.access("/community/object_ml/")
        category = re.compile('<a href="http://[^"]*(/community/object_ml/./[^?"]*)"')
        return re.findall(category, page)

    def list_entry_urls(self, category_url):
        page = self.access(category_url)
        block_pattern = re.compile('<h1 class="documentFirstHeading">.*<!-- end of main content block -->', re.DOTALL)

        m = block_pattern.search(page)
        block = page[m.start():m.end()]
        url_pattern = re.compile('<a href="[^"]*(/community/object_ml/./([^"/]*))">([^<]*)</a>')
        matches = url_pattern.finditer(block)
        for m in matches:
            yield m.groups()

    def list_entries(self):
        category_urls = self.list_category_urls()
        for cat in category_urls:
            for url in self.list_entry_urls(cat):
                yield url

    def read_evaluation(self, url):
        page = self.access(url + "/PlonePopoll_results2")
        block_pattern = re.compile('<div class="documentContent" id="region-content">' +
                                   '.*' + 
                                   '<!-- end of main content block -->', re.DOTALL)
        m = block_pattern.search(page)
        block = page[m.start():m.end()]

        url_pattern = re.compile('<tr>\s*<td>([^<]*)</td>\s*' +
                                 '<td>\s*<img[^>]*>\s*[^<]*\s*</td>\s*' +
                                 '<td>([^<]*)%\s*</td>\s*' +
                                 '<td>\(([^)]*) votes\)</td>\s*</tr>', re.DOTALL)
        matches = url_pattern.finditer(block)
        results = {}
        for m in matches:
            name, percent, votes = m.groups()
            results[name] = int(votes)
        return results

    def get_all_evaluations(self):
        entries = self.list_entries()
        result = Result(datetime.datetime.today())
        result.entries = (Entry(e[1], e[2], self.read_evaluation(e[0])) for e in entries)
        return result

def main():
    site = ObLoveSite("http://www.objectclub.jp:80")


if __name__ == '__main__':
    main()


