﻿# coding: utf-8
import sys
import os
import unittest
from table import *

class TableTest(unittest.TestCase):
    def setUp(self):
        pass

    def test_table_empty(self):
        t = Table()
        self.assertEqual(0, t.row_count())

    def test_table_value3by3(self):
        t = Table()
        t.add_cell(1)
        t.add_cell(2)
        t.add_cell(3)
        t.add_row()
        t.add_cell(4)
        t.add_cell(5)
        t.add_cell(6)
        t.add_row()
        t.add_cell(7)
        t.add_cell(8)
        t.add_cell(9)
        t.add_row()

        self.assertEqual(3, t.row_count())
        self.assertEqual(3, t.col_count(0))
        self.assertEqual(1, t.value(0, 0))
        self.assertEqual(2, t.value(1, 0))
        self.assertEqual(3, t.value(2, 0))
        self.assertEqual(3, t.col_count(1))
        self.assertEqual(4, t.value(0, 1))
        self.assertEqual(5, t.value(1, 1))
        self.assertEqual(6, t.value(2, 1))
        self.assertEqual(3, t.col_count(2))
        self.assertEqual(7, t.value(0, 2))
        self.assertEqual(8, t.value(1, 2))
        self.assertEqual(9, t.value(2, 2))

    def test_table_cvs3by3(self):
        t = Table()
        t.add_cell(1)
        t.add_cell(2)
        t.add_cell(3)
        t.add_row()
        t.add_cell(4)
        t.add_cell(5)
        t.add_cell(6)
        t.add_row()
        t.add_cell(7)
        t.add_cell(8)
        t.add_cell(9)
        t.add_row()

        expected = ""
        expected += "1,2,3\n"
        expected += "4,5,6\n"
        expected += "7,8,9\n"
        self.assertEqual(expected, t.csv())

    def test_table_properties(self):
        t = Table()
        c = t.add_cell(1)
        c.color = 'red'
        c = t.add_cell(2)
        c.color = 'blue'
        c.style = 'strong'
        t.add_row()
        c = t.add_cell(3)
        c.style = 'strongest'
        c = t.add_cell(4)
        t.add_row()

        self.assertEqual('red', t.get_cell(0, 0).color)
        self.assertEqual('blue', t.get_cell(1, 0).color)
        self.assertEqual('strong', t.get_cell(1, 0).style)
        self.assertEqual('strongest', t.get_cell(0, 1).style)

    def test_table_add_concurrently(self):
        t = Table()
        r1 = t.get_current_row()
        t.add_row()
        r2 = t.get_current_row()
        t.add_row()
        r3 = t.get_current_row()
        t.add_row()
        r1.add_cell(1)
        r2.add_cell(4)
        r3.add_cell(7)
        r1.add_cell(2)
        r2.add_cell(5)
        r3.add_cell(8)
        r1.add_cell(3)
        r2.add_cell(6)
        r3.add_cell(9)
        
        self.assertEqual(3, t.row_count())
        self.assertEqual(1, t.value(0, 0))
        self.assertEqual(2, t.value(1, 0))
        self.assertEqual(3, t.value(2, 0))
        self.assertEqual(4, t.value(0, 1))
        self.assertEqual(5, t.value(1, 1))
        self.assertEqual(6, t.value(2, 1))
        self.assertEqual(7, t.value(0, 2))
        self.assertEqual(8, t.value(1, 2))
        self.assertEqual(9, t.value(2, 2))


if __name__ == '__main__':
    unittest.main()


